using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using Upsilon.Evaluator;

namespace Upsilon.BaseTypes.UserData
{
    public class UserDataMethod
    {
        public class UserDataMethodPart
        {
            public UserDataMethodPart(MethodInfo method)
            {
                Method = method;
                var pars = method.GetParameters();
                var ls = new List<UserDataMethodParameter>();
                foreach (var parameter in pars)
                {
                    if (parameter.ParameterType == typeof(Script))
                        continue;
                    if (parameter.ParameterType == typeof(EvaluationScope))
                        continue;
                    ls.Add(new UserDataMethodParameter(parameter));
                }
                Parameters = ls.ToArray();
            }

            public MethodInfo Method { get; }
            public UserDataMethodParameter[] Parameters { get; }
        }

        public struct UserDataMethodParameter
        {
            public UserDataMethodParameter(ParameterInfo info)
            {
                Type = info.ParameterType;
                IsOptional = info.IsOptional;
            }

            public System.Type Type { get; }
            public bool IsOptional { get; }
        }

        public string Name { get; }
        private List<UserDataMethodPart> MethodParts { get; }
        public System.Type ReturnType { get; }

        public UserDataMethod(MethodInfo method)
        {
            Name = method.Name;
            var part = new UserDataMethodPart(method);
            MethodParts = new List<UserDataMethodPart>()
            {
                part
            };
            ReturnType = part.Method.ReturnType;
        }

        public void LoadMethodPart(MethodInfo method)
        {
            var part = new UserDataMethodPart(method);
            MethodParts.Add(part);
        }

        public List<UserDataMethodPart> GetMethods()
        {
            return MethodParts;
        }

        public MethodInfo GetMethod(System.Type[] parameterTypes)
        {
            foreach (var userDataMethodPart in MethodParts)
            {
                bool valid = true;
                for (var index = 0; index < userDataMethodPart.Parameters.Length; index++)
                {
                    var userDataMethodParameter = userDataMethodPart.Parameters[index];
                    if (index >= parameterTypes.Length)
                    {
                        if (userDataMethodParameter.IsOptional)
                            return userDataMethodPart.Method;
                        valid = false;
                        break;
                    }
                    var parameter = parameterTypes[index];
                    if (userDataMethodParameter.Type != parameter &&
                        !userDataMethodParameter.Type.IsAssignableFrom(parameter))
                    {
                        valid = false;
                        break;
                    }
                }

                if (valid) return userDataMethodPart.Method;
            }

            return null;
        }
    }
}