using System.Collections;
using System.Collections.Generic;
using System.Linq;
using Upsilon.BaseTypes;

namespace Upsilon.BoundTypes
{
    public static class BoundTypeHandler
    {
        private static readonly Dictionary<string, BoundTypeDefinition> DefaultTypeDefinitions =
            new Dictionary<string, BoundTypeDefinition>
        {
            {"string", new BoundTypeDefinition(Type.String, typeof(string))},
            {
                "number",
                new BoundTypeDefinition(Type.Number,
                    new[] {typeof(int), typeof(long), typeof(float), typeof(double)})
            },
            {"bool", new BoundTypeDefinition(Type.Boolean, typeof(bool))},
            {"table", new BoundTypeDefinition(Type.Table, typeof(IEnumerator))},
            {"function", new BoundTypeDefinition(Type.Function, new System.Type[0])},
        };

        private static Dictionary<string, BoundTypeDefinition> _typeDefinitions =
            new Dictionary<string, BoundTypeDefinition>(DefaultTypeDefinitions);

        public static void Reset()
        {
            _typeDefinitions =
                new Dictionary<string, BoundTypeDefinition>(DefaultTypeDefinitions);
        }

        public static BoundTypeDefinition GetTypeDefinition(string key)
        {
            var normalizedName = key.ToLowerInvariant();
            if (_typeDefinitions.TryGetValue(normalizedName, out var bt))
            {
                return bt;
            }
            return null;
        }
        public static BoundTypeDefinition GetTypeDefinition(System.Type type)
        {
            return _typeDefinitions.Values.FirstOrDefault(x => x.ValidInternalTypes.Contains(type));
        }

        public static void LoadUserDataTypeDefinition(UserDataBoundTypeDefinition def)
        {
            var key = def.Name.ToLowerInvariant();
            if (_typeDefinitions.ContainsKey(key))
            {
                _typeDefinitions[key] = def;
            }
            else
            {
                _typeDefinitions.Add(key, def);
            }
        }
    }
}