namespace Upsilon.Parser
{
    public static class SyntaxKindPrecedence
    {
        public enum Precedence
        {
            None = 0,
            Or,
            And,
            Equality,
            PlusMinus,
            StarSlash,
            Unary,
            Exponentiation
        }

        public static Precedence UnaryOperatorPrecedence(this SyntaxKind kind)
        {
            switch (kind)
            {
                case SyntaxKind.Plus:
                case SyntaxKind.Minus:
                case SyntaxKind.NotKeyword:
                case SyntaxKind.PoundSign:
                    return Precedence.Unary;
                default:
                    return Precedence.None;
            }
        }

        public static Precedence BinaryOperatorPrecedence(this SyntaxKind kind)
        {
            switch (kind)
            {
                // equality operators
                case SyntaxKind.EqualsEquals:
                case SyntaxKind.TildeEquals:

                case SyntaxKind.Greater:
                case SyntaxKind.GreaterEquals:
                case SyntaxKind.Less:
                case SyntaxKind.LessEquals:
                    return Precedence.Equality;

                // logical operators
                case SyntaxKind.AndKeyword:
                    return Precedence.And;
                case SyntaxKind.OrKeyword:
                    return Precedence.Or;

                // math operators
                case SyntaxKind.Plus:
                case SyntaxKind.Minus:
                    return Precedence.PlusMinus;

                case SyntaxKind.Star:
                case SyntaxKind.Slash:
                    return Precedence.StarSlash;
                default:
                    return Precedence.None;
            }

        }
    }
}